@@ -46,7 +46,6 @@
 #include "champlain-zoom-level.h"
 
 #include <clutter/clutter.h>
-#include <clutter-cairo/clutter-cairo.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <cairo.h>
@@ -55,6 +54,8 @@
 
 #define DEFAULT_FONT_NAME "Sans 11"
 
+static ClutterColor SELECTED_COLOR = {0x00, 0x33, 0xcc, 0xff};
+
 static ClutterColor DEFAULT_COLOR = {0x33, 0x33, 0x33, 0xff};
 static ClutterColor DEFAULT_TEXT_COLOR = {0xee, 0xee, 0xee, 0xff};
 
@@ -113,6 +114,24 @@ G_DEFINE_TYPE (ChamplainMarker, champlain_marker, CHAMPLAIN_TYPE_BASE_MARKER);
 static void draw_marker (ChamplainMarker *marker);
 static void queue_redraw (ChamplainMarker *marker);
 
+/**
+ * champlain_marker_set_hightlight_color:
+ * @color: a #ClutterColor
+ *
+ * Changes the highlight color, this is to ensure a better integration with
+ * the desktop, this is automatically done by GtkChamplainEmbed.
+ *
+ * Since: 0.2
+ */
+void
+champlain_marker_set_highlight_color (ClutterColor *color)
+{
+  SELECTED_COLOR.red = color->red;
+  SELECTED_COLOR.green = color->green;
+  SELECTED_COLOR.blue = color->blue;
+  SELECTED_COLOR.alpha = color->alpha;
+}
+
 static void
 champlain_marker_get_property (GObject *object,
                                guint prop_id,
@@ -448,8 +467,8 @@ draw_shadow (ChamplainMarker *marker,
   else
     x = -58 * slope;
 
-  shadow = clutter_cairo_new (width + x, (height + point));
-  cr = clutter_cairo_create (CLUTTER_CAIRO (shadow));
+  shadow = clutter_cairo_texture_new (width + x, (height + point));
+  cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (shadow));
 
   cairo_matrix_init (&matrix,
       1, 0,
@@ -486,22 +505,31 @@ draw_background (ChamplainMarker *marker,
     gint point)
 {
   ChamplainMarkerPrivate *priv = marker->priv;
+  ChamplainBaseMarkerPrivate *base_priv = CHAMPLAIN_BASE_MARKER (marker)->priv;
   ClutterActor *bg = NULL;
+  ClutterColor *color;
   ClutterColor darker_color;
   cairo_t *cr;
 
-  bg = clutter_cairo_new (width, height + point);
-  cr = clutter_cairo_create (CLUTTER_CAIRO (bg));
+  bg = clutter_cairo_texture_new (width, height + point);
+  cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (bg));
+
+  /* If selected, add the selection color to the marker's color */
+  if (base_priv->highlighted)
+    color = &SELECTED_COLOR;
+  else
+    color = priv->color;
+
 
   draw_box (cr, width, height, point, priv->alignment == PANGO_ALIGN_LEFT);
 
-  clutter_color_darken (priv->color, &darker_color);
+  clutter_color_darken (color, &darker_color);
 
   cairo_set_source_rgba (cr,
-      priv->color->red / 255.0,
-      priv->color->green / 255.0,
-      priv->color->blue / 255.0,
-      priv->color->alpha / 255.0);
+      color->red / 255.0,
+      color->green / 255.0,
+      color->blue / 255.0,
+      color->alpha / 255.0);
   cairo_fill_preserve (cr);
 
   cairo_set_line_width (cr, 1.0);
@@ -543,22 +571,22 @@ draw_marker (ChamplainMarker *marker)
 
   if (priv->text != NULL && strlen (priv->text) > 0)
     {
-      ClutterLabel *label;
+      ClutterText *label;
       if (priv->text_actor == NULL)
       {
-        priv->text_actor = clutter_label_new_with_text (priv->font_name, priv->text);
+        priv->text_actor = clutter_text_new_with_text (priv->font_name, priv->text);
         g_object_ref (priv->text_actor);
       }
 
-      label = CLUTTER_LABEL (priv->text_actor);
-      clutter_label_set_use_markup (label, priv->use_markup);
-      clutter_label_set_font_name (label, priv->font_name);
-      clutter_label_set_text (label, priv->text);
-      clutter_label_set_alignment (label, priv->alignment);
-      clutter_label_set_line_wrap (label, priv->wrap);
-      clutter_label_set_line_wrap_mode (label, priv->wrap_mode);
-      clutter_label_set_ellipsize (label, priv->ellipsize);
-      clutter_label_set_attributes (label, priv->attributes);
+      label = CLUTTER_TEXT (priv->text_actor);
+      clutter_text_set_font_name (label, priv->font_name);
+      clutter_text_set_text (label, priv->text);
+      clutter_text_set_line_alignment (label, priv->alignment);
+      clutter_text_set_line_wrap (label, priv->wrap);
+      clutter_text_set_line_wrap_mode (label, priv->wrap_mode);
+      clutter_text_set_ellipsize (label, priv->ellipsize);
+      clutter_text_set_attributes (label, priv->attributes);
+      clutter_text_set_use_markup (label, priv->use_markup);
 
       height = clutter_actor_get_height (priv->text_actor);
       if (priv->image != NULL)
@@ -576,7 +604,7 @@ draw_marker (ChamplainMarker *marker)
       if (height > total_height)
         total_height = height;
 
-      clutter_label_set_color (CLUTTER_LABEL (priv->text_actor), priv->text_color);
+      clutter_text_set_color (CLUTTER_TEXT (priv->text_actor), priv->text_color);
       if (clutter_actor_get_parent (priv->text_actor) == NULL)
         clutter_container_add_actor (CLUTTER_CONTAINER (marker), priv->text_actor);
     }
@@ -649,6 +677,14 @@ queue_redraw (ChamplainMarker *marker)
     marker->priv->redraw_id = g_idle_add (redraw_on_idle, marker);
 }
 
+static void
+notify_highlighted (GObject *gobject,
+    GParamSpec *pspec,
+    gpointer user_data)
+{
+  queue_redraw (CHAMPLAIN_MARKER (gobject));
+}
+
 static void
 champlain_marker_init (ChamplainMarker *marker)
 {
@@ -670,6 +706,8 @@ champlain_marker_init (ChamplainMarker *marker)
   priv->ellipsize = PANGO_ELLIPSIZE_NONE;
   priv->draw_background = TRUE;
   priv->redraw_id = 0;
+
+  g_signal_connect (marker, "notify::highlighted", G_CALLBACK (notify_highlighted), NULL);
 }
 
 /**
